#!/usr/local/bin/perl

#
# install.pl
#
# installs Ecescan into the system
#
# chok, last updated on mon 11 jun 2001
#

$CFG_SCRIPT = './install.cfg.pl';
$REAL_CFG   = './ecescan.cfg';


#
# check for configuration script
#

unless( -f $CFG_SCRIPT )
{
	print( "\nCannot find the install-config script: $CFG_SCRIPT\n\n" );
	print( "Please make sure you are running the script from the\n" );
	print( "same directory as all the rest of the system files\n\n" );
	exit( 1 );
}

#
# read the config script
#

chmod( 0644, $CFG_SCRIPT );
do( $CFG_SCRIPT );

if( $! || $@ )
{
	print( "\nCannot read the install-config script: $CFG_SCRIPT\n\n" );
	print( "Error: $!$@\n\n" );	# prints the non-empty one
	exit( 1 );
}

#
# check that all directories are defined properly
#

$missingFlag = 0;
dirMissing( 'ECESCAN_DIR'         ) if( length( $ECESCAN_DIR ) == 0 );
dirMissing( 'ECESCAN_CFG_DIR'     ) if( length( $ECESCAN_CFG_DIR ) == 0 );
dirMissing( 'ECESCAN_LOG_DIR'     ) if( length( $ECESCAN_LOG_DIR ) == 0 );
dirMissing( 'ECESCAN_OUT_DIR'     ) if( length( $ECESCAN_OUT_DIR ) == 0 );
dirMissing( 'ECESCAN_WWW_DIR'     ) if( length( $ECESCAN_WWW_DIR ) == 0 );
dirMissing( 'ECESCAN_WWW_OUT_DIR' ) if( length( $ECESCAN_WWW_OUT_DIR ) == 0 );
dirMissing( 'ECESCAN_CGI_DIR'     ) if( length( $ECESCAN_CGI_DIR ) == 0 );
dirMissing( 'ECESCAN_BK_DIR'      ) if( length( $ECESCAN_BK_DIR ) == 0 );
dirMissing( 'ECESCAN_DNS_DIR'     ) if( length( $ECESCAN_DNS_DIR ) == 0 );

sub dirMissing # args: $dirName
# prints short message saying directory is not there and sets $missingFlag
{
	print( "Missing directory in $CFG_SCRIPT: $_[ 0 ]\n" );
	$missingFlag = 1;
}

if( $missingFlag == 1 )
{
	print( "\nOne or more directories were missing from $CFG_SCRIPT.\n" );
	print( "Please include the missing directories before running install.pl again.\n\n" );
	exit( 1 );
}

#
# prompt user to continue
#

print( "\nEcescan Installation ... \n\n" );

print( "Copyright 2000-2002\n" );
print( "Ecescan is an intellectual property of the School of ECE,\n" );
print( "Georgia Institute of Technology, Atlanta, Georgia\n\n" );

print( "This installation will automate the entire installation process.\n" );
print( "Here are the directories that are specified in $CFG_SCRIPT in which\n" );
print( "the entire Ecescan system will be installed:\n\n" );

print( "Main       - $ECESCAN_DIR\n" );
print( "Config     - $ECESCAN_CFG_DIR\n" );
print( "Log        - $ECESCAN_LOG_DIR\n" );
print( "Output     - $ECESCAN_OUT_DIR\n" );
print( "WWW        - $ECESCAN_WWW_DIR\n" );
print( "WWW Output - $ECESCAN_WWW_OUT_DIR\n" );
print( "CGI        - $ECESCAN_CGI_DIR\n" );
print( "Backup     - $ECESCAN_BK_DIR\n" );
print( "DNS        - $ECESCAN_DNS_DIR\n\n" );

print( "If you want to install using the above directories, please enter 'y'.\n" );
print( "Is it okay to install using these directories? " );

$input = <STDIN>; chomp( $input );
if( $input ne 'y' && $input ne 'Y' )
{
	print( "\n", '-' x 60, "\n" );
	print( "** Installation Abandoned **\n\n" );
	print( "Please edit the file $CFG_SCRIPT to indicate the directories\n" );
	print( "before you try to install again.\n\n" );
	exit( 1 );
}


#
# create dir if not exists all
#

print( "\nCreating all directories ...\n" );

ensureDir( $ECESCAN_DIR );
ensureDir( $ECESCAN_CFG_DIR );
ensureDir( $ECESCAN_LOG_DIR );
ensureDir( $ECESCAN_OUT_DIR );
ensureDir( $ECESCAN_WWW_DIR );
ensureDir( $ECESCAN_WWW_OUT_DIR );
ensureDir( $ECESCAN_CGI_DIR );
ensureDir( $ECESCAN_BK_DIR );
ensureDir( $ECESCAN_DNS_DIR );
ensureDir( $ECESCAN_SRC_DIR = $ECESCAN_DIR . '/src' );	# for source scripts

sub ensureDir # args: $fullPath
# creates the directory if it does not exist
{
	my( @subDirs ) = split( /\//, $_[ 0 ] );
	my( $dir ) = '';

	for $s ( @subDirs ) {

		next if length( $s ) == 0;

		$dir .= '/' . $s;

		unless( -d $dir ) {
			unless( mkdir( $dir, 0700 ) ) {
				print( "\nCannot create dir $dir: $!\n\n" );
				print( "Please check to make sure the file permissions for the parent\n" );
				print( "dir of this dir allows you to write to that parent dir\n\n" );
				exit( 1 );
				}
			}

		}
}

#
# now, replace the dirs in the real configuration file with the dirs in install.cfg.pl
#

$tmp = $REAL_CFG . '.tmp';
unlink( $tmp ) if( -f $tmp );

unless( open( INFH, "< $REAL_CFG" ) && open( OUTFH, "> $tmp" ) )
{
	print( "\nCannot read ecescan main configuration file: $REAL_CFG\n\n" );
	print( "Error: $!\n\n" );
	exit( 1 );
}

print( "\nConfiguring main config file $REAL_CFG ...\n" );

while( $line = <INFH> )
{
	if( $line =~ m/^\s*\$ECESCAN_DIR\s*=\s*["'][^"']+["']\s*;/         ) { $line = "\$ECESCAN_DIR = '$ECESCAN_DIR';\n"; }
	if( $line =~ m/^\s*\$ECESCAN_CFG_DIR\s*=\s*["'][^"']+["']\s*;/     ) { $line = "\$ECESCAN_CFG_DIR = '$ECESCAN_CFG_DIR';\n"; }
	if( $line =~ m/^\s*\$ECESCAN_LOG_DIR\s*=\s*["'][^"']+["']\s*;/     ) { $line = "\$ECESCAN_LOG_DIR = '$ECESCAN_LOG_DIR';\n"; }
	if( $line =~ m/^\s*\$ECESCAN_OUT_DIR\s*=\s*["'][^"']+["']\s*;/     ) { $line = "\$ECESCAN_OUT_DIR = '$ECESCAN_OUT_DIR';\n"; }
	if( $line =~ m/^\s*\$ECESCAN_WWW_DIR\s*=\s*["'][^"']+["']\s*;/     ) { $line = "\$ECESCAN_WWW_DIR = '$ECESCAN_WWW_DIR';\n"; }
	if( $line =~ m/^\s*\$ECESCAN_WWW_OUT_DIR\s*=\s*["'][^"']+["']\s*;/ ) { $line = "\$ECESCAN_WWW_OUT_DIR = '$ECESCAN_WWW_OUT_DIR';\n"; }
	if( $line =~ m/^\s*\$ECESCAN_CGI_DIR\s*=\s*["'][^"']+["']\s*;/     ) { $line = "\$ECESCAN_CGI_DIR = '$ECESCAN_CGI_DIR';\n"; }
	if( $line =~ m/^\s*\$ECESCAN_BK_DIR\s*=\s*["'][^"']+["']\s*;/      ) { $line = "\$ECESCAN_BK_DIR = '$ECESCAN_BK_DIR';\n"; }
	if( $line =~ m/^\s*\$ECESCAN_DNS_DIR\s*=\s*["'][^"']+["']\s*;/     ) { $line = "\$ECESCAN_DNS_DIR = '$ECESCAN_DNS_DIR';\n"; }

	print OUTFH $line;
}

close( INFH ); close( OUTFH );

$org = $REAL_CFG . '.org';
unlink( $org ) if( -f $org );

unless( rename( $REAL_CFG, $org ) )
{
	print( "\nCannot rename $REAL_CFG to $org: $!\n\n" );
	exit( 0 );
}

unless( rename( $tmp, $REAL_CFG ) )
{
	print( "\nCannot rename $tmp to $REAL_CFG: $!\n\n" );
	exit( 0 );
}

unlink( $tmp );

#
# everything ok! now, copy the files
#

# only 3 different directories to copy any files into
# (ignore errors)

print( "\nCopying all files ...\n" );

if(	system( "cp ../www/* $ECESCAN_WWW_DIR" )
	|| system( "cp ../src/* $ECESCAN_SRC_DIR" )
	|| system( "cp ../cfg/* $ECESCAN_CFG_DIR" )
  )
{
	print( "\nCannot copy files to destination dir: $?\n\n" );
	exit( 1 );
}

#
# make all scripts
#

print( "\nMaking all shortcut scripts\n" );

# replace line in 'makeexe': $ecescanCfg = '...';

$makeexe = "$ECESCAN_SRC_DIR/makeexe";
$tmp = $makeexe . '.tmp';
unlink( $tmp ) if( -f $tmp );

unless( open( INFH, "< $makeexe" ) && open( OUTFH, "> $tmp" ) )
{
	print( "\nCannot read $makeexe: $!\n\n" );
	print( "Error: $!\n\n" );
	exit( 1 );
}

while( $line = <INFH> )
{
	if( $line =~ m/\$ecescanCfg\s*=\s*'[^']+';/ ) { $line = "\$ecescanCfg = '$ECESCAN_CFG_DIR/ecescan.cfg';\n"; }
	print OUTFH $line;
}

close( $INFH ); close( OUTFH ); 

$org = $makeexe . '.org';
unlink( $org ) if( -f $org );

unless( rename( $makeexe, $org ) )
{
	print( "\nCannot rename $makeexe to $org: $!\n\n" );
	exit( 0 );
}

unless( rename( $tmp, $makeexe ) )
{
	print( "\nCannot rename $tmp to $makeexe: $!\n\n" );
	exit( 0 );
}

unlink( $tmp );

unless( chmod( 0755, $makeexe ) )
{
	print( "\nCannot chmod 755 '$makeexe': $!\n\n" );
	exit( 1 );
}

if( system( $makeexe ) )
{
	print( "\nCannot create shortcut scripts\n\n" );
	exit( 1 );
}

#
# set file permissions
#

print( "\nSetting all file permissions ...\n" );

# replace line in 'chmodall': $ecescanCfg = '...';

$chmodall = "$ECESCAN_SRC_DIR/chmodall";
$tmp = $chmodall . '.tmp';
unlink( $tmp ) if( -f $tmp );

unless( open( INFH, "< $chmodall" ) && open( OUTFH, "> $tmp" ) )
{
	print( "\nCannot read $chmodall: $!\n\n" );
	print( "Error: $!\n\n" );
	exit( 1 );
}

while( $line = <INFH> )
{
	if( $line =~ m/\$ecescanCfg\s*=\s*'[^']+';/ ) { $line = "\$ecescanCfg = '$ECESCAN_CFG_DIR/ecescan.cfg';\n"; }
	print OUTFH $line;
}

close( $INFH ); close( OUTFH ); 

$org = $chmodall . '.org';
unlink( $org ) if( -f $org );

unless( rename( $chmodall, $org ) )
{
	print( "\nCannot rename $chmodall to $org: $!\n\n" );
	exit( 0 );
}

unless( rename( $tmp, $chmodall ) )
{
	print( "\nCannot rename $tmp to $chmodall: $!\n\n" );
	exit( 0 );
}

unlink( $tmp );

unless( chmod( 0755, $chmodall ) )
{
	print( "\nCannot chmod 755 '$chmodall': $!\n\n" );
	exit( 1 );
}

if( system( "$ECESCAN_DIR/chmodall" ) )
{
	print( "\nCannot set all file's permissions\n\n" );
	exit( 1 );
}

#
# fix ecescan.data
#

print( "\nConfiguring data file 'ecescan.data' ...\n" );

# replace line in 'ecescan.data': $ECESCAN_CFG = '...';

$data = "$ECESCAN_SRC_DIR/ecescan.data";
$tmp = $data . '.tmp';
unlink( $tmp ) if( -f $tmp );

unless( open( INFH, "< $data" ) && open( OUTFH, "> $tmp" ) )
{
	print( "\nCannot read $data: $!\n\n" );
	print( "Error: $!\n\n" );
	exit( 1 );
}

while( $line = <INFH> )
{
	if( $line =~ m/\$ECESCAN_CFG\s*=\s*'[^']+';/ ) { $line = "\$ecescanCfg = '$ECESCAN_CFG_DIR/ecescan.cfg';\n"; }
	print OUTFH $line;
}

close( $INFH ); close( OUTFH ); 

$org = $data . '.org';
unlink( $org ) if( -f $org );

unless( rename( $data, $org ) )
{
	print( "\nCannot rename $data to $org: $!\n\n" );
	exit( 0 );
}

unless( rename( $tmp, $data ) )
{
	print( "\nCannot rename $tmp to $data: $!\n\n" );
	exit( 0 );
}

unlink( $tmp );


#
# success!
#

print( "\n** Installation completed! **\n\n" );

print( "The database has not been created in the case that you already\n" );
print( "have Ecescan running and so your data will not be deleted.\n\n" );

print( "There are various configurations settings that you may specify in\n" );
print( "the configuration file '$ECESCAN_CFG_DIR/ecescan.cfg'\n" );
print( "before you try running the script 'ecescan'. Otherwise, the script\n" );
print( "might not work correctly on this machine using our default settings.\n\n" );

print( "Good luck!\n\n" );



# end of file install.pl