#!/usr/local/bin/perl

# update_dns.pl: chok(last updated on tue 9 jul 2002)

use lib '.';
use ecescan;

do( './ecescan.data' );
do( $ECESCAN_CFG );



sub update_dns # no args
# create the data table 'dns_comparisons'
# as (ip, hostname, disagreement, based_on, long_ip)
{
	# vars
	$dns				= 'DNS file';
	$ece				= 'Ecescan data';

	$temp_table_fields = '( ip char(15), hostname varchar(255),
		long_ip int unsigned )';

	$table_fields = '( ip char(15), hostname varchar(255),
		disagreement varchar(255), based_on char(3),
		long_ip int unsigned not null )';

	# database init
	SQL_do("drop table if exists dnstemp");
	SQL_do("drop table if exists ecetemp");
	SQL_do("drop table if exists dns_comparisons");
	SQL_do("create table dnstemp $temp_table_fields");
	SQL_do("create table ecetemp $temp_table_fields");
	SQL_do("create table dns_comparisons $table_fields");












	# store all matching IP in all dns files into database
	$count_dns = 0;

	# get IP -> DNS filename mapping
	require('./dns_filenames.pl');

	# read from DNS file(s)
	while( ($subnet,$dnsfile) = each(%ip2file_map) ) {

		$dnsfile = "$ECESCAN_DNS_DIR/$dnsfile";

		# read entire file as an array
		my @f = ();
		if(! -f $dnsfile)
			{print("DNS file \"$dnsfile\" not found<br><br><br>");}
		elsif(! -r $dnsfile)
			{print("DNS file \"$dnsfile\" is not readable<br><br><br>");}
		else {
			# read in entire file
			open(FH,$dnsfile) or die($dnsfile);
			while($s=<FH>) {push(@f,$s);}
			close(FH);
		}

		$lineno = 0;
		foreach $s (@f) {

			$lineno ++;

			# format in DNS file: 1 in ptr hostname.ece.gatech.edu
			# the first number is the xxx in 123.123.123.xxx

			# ignore line if not an IP line
			next unless $s =~ m/^([0-9]+)\s+in\s+ptr\s+([^\s]+)/;

			# check ip address for validity
			if( $1<0 or $1>255 ) {
				print("error: DNS file \"$dnsfile\": line $lineno: " .
					"invalid IP sub-address \"$1\" (only 0-255 valid)");
				exit(1);
			}

			# read matched DNS data line
			$ip			= "$subnet.$1";
			$hostname	= $2;
			$long_ip	= ip_to_long($ip);

			# remove trailing '.' in hostname
			chop($hostname) if substr($hostname,length($hostname)-1,1) eq '.';

			# write info into DNS table
			SQL_do("insert into dnstemp (ip,hostname,long_ip) ".
				"values('$ip', '$hostname', '$long_ip')");

			$count_dns ++;

		} #foreach

	} #while












	# store all matching IP in all ecescan data into database table
	$count_ece = 0;

	$sql = "select ip, hostname, longip from iptable where " .
		"ip regexp '(" . join(")|(", keys(%ip2file_map)) . ")'";
	SQL_prepare_execute($sql);
	while(@a=SQL_fetchrow()) {
		$count_ece++;
		($ip, $hostname, $long_ip) = @a;

		SQL_do qq{ insert into ecetemp (ip,hostname,long_ip)
			values('$ip', '$hostname', '$long_ip') };
	}
	SQL_finishstatement();












	# cycle thru each IP in ecetemp
	$sql = "select ip, hostname, long_ip from ecetemp order by long_ip";
	SQL_prepare_execute($sql);
	while(@a=SQL_fetchrow()) {

		($ip,$hostname,$long_ip) = @a;
		$disagreement = $semi = '';


		# check whether this IP exists in DNS data
		$sql = "select hostname from dnstemp where ip='$ip'";
		SQL_prepare_execute1($sql);
		my @b = SQL_fetchrow1();
		SQL_finishstatement1();

		$ip_exists = scalar(@b);
		if($ip_exists) { # IP exists in ecetemp as well
			if($b[0] ne $hostname) { # different hostname
				$disagreement = "same IP, different hostname \"$b[0]\"";
				$semi = '; ';
			}
		}


		# check whether this Hostname exists in DNS data
		# note: we search thru the 'iptable' again
		$sql = "select ip from dnstemp where hostname='$hostname'";
		SQL_prepare_execute1($sql);
		while( @b = SQL_fetchrow1() ) {
			if(scalar(@b)) { # Hostname exists in dnstemp as well
				$hostname_exists = 1;

				if($b[0] ne $ip) { # different IP
					$disagreement .= "${semi}different IP \"$b[0]\" with same hostname";
					$semi = '; ';
				}
			}
		}
		SQL_finishstatement1();


		# check whether this entry is totally not found or what
		$flag = 1;
		if(!$ip_exists && !$hostname_exists) {
			$disagreement = "This IP-Hostname pair is " .
				"found in the $ece, but both the IP and the " .
				"hostname individually do not occur anywhere in the $dns";
			$flag = 0;
		}

		# insert data into disagreements table
		if(length($disagreement)) {
			if($flag)
				{$disagreement = "Found in $ece: Conflicts with $dns: $disagreement";}

			SQL_do qq{ insert into dns_comparisons
				(ip,hostname,disagreement,based_on,long_ip)
				values('$ip','$hostname','$disagreement','ECE','$long_ip')
			};
		}

	}
	SQL_finishstatement();















	# cycle thru each IP in dnstemp

	# ONLY EXTRACT THOSE THAT ARE TOTALLY MISSING FROM ECESCAN DATA

	$sql = "select ip, hostname, long_ip from dnstemp order by long_ip";
	SQL_prepare_execute($sql);
	while(@a=SQL_fetchrow()) {

		($ip,$hostname,$long_ip) = @a;


		# check whether this IP exists in ecescan data
		# note: we search in ecetemp and not iptable becos if it is
		#       found in iptable, then it will be in ecetemp
		$sql = "select hostname from ecetemp where ip='$ip'";
		SQL_prepare_execute1($sql);
		my @b = SQL_fetchrow1();
		SQL_finishstatement1();
		$ip_exists = scalar(@b);


		# check whether this Hostname exists in ecescan data
		# note: we search thru the 'iptable' again
		$sql = "select ip from iptable where hostname='$hostname'";
		SQL_prepare_execute1($sql);
		@b = SQL_fetchrow1();
		SQL_finishstatement1();
		$hostname_exists = scalar(@b);


		# check whether this entry is totally not found or what
		if(!$ip_exists && !$hostname_exists) {
			$disagreement = "This IP-Hostname pair is " .
				"found in the $dns, but both the IP and the " .
				"hostname individually do not occur anywhere in the $ece";

			# insert data into final results table
			SQL_do qq{ insert into dns_comparisons
				(ip,hostname,disagreement,based_on,long_ip)
				values('$ip','$hostname','$disagreement','DNS','$long_ip')
			};
		}

	}
	SQL_finishstatement();


















	SQL_do("drop table dnstemp");
	SQL_do("drop table ecetemp");

	return 1; # success!
}

connectdatabase();

if(update_dns())
	{print("All DNS data updated successfully\n");}
else
	{print("update_dns.pl failed\n");}

enddatabase();



1;
